@@ -14,8 +14,6 @@
 // You should have received a copy of the GNU General Public License
 // along with the snarkVM library. If not, see <https://www.gnu.org/licenses/>.
 
-pub use crate::crh::pedersen_parameters::PedersenSize;
-
 use super::bowe_hopwood_pedersen_parameters::*;
 use crate::{
     crh::{PedersenCRH, PedersenCRHParameters},
@@ -31,21 +29,21 @@ use rand::Rng;
 
 // we cant use these in array sizes since they are from a trait (and cant be refered to at const time)
 const MAX_WINDOW_SIZE: usize = 256;
-const MAX_NUM_WINDOWS: usize = 296;
+const MAX_NUM_WINDOWS: usize = 4096;
 
 #[derive(Debug, Clone, PartialEq, Eq)]
-pub struct BoweHopwoodPedersenCRH<G: Group, S: PedersenSize> {
-    pub parameters: PedersenCRHParameters<G, S>,
+pub struct BoweHopwoodPedersenCRH<G: Group, const NUM_WINDOWS: usize, const WINDOW_SIZE: usize> {
+    pub parameters: PedersenCRHParameters<G, NUM_WINDOWS, WINDOW_SIZE>,
     pub bowe_hopwood_parameters: BoweHopwoodPedersenCRHParameters<G>,
 }
 
-impl<G: Group, S: PedersenSize> BoweHopwoodPedersenCRH<G, S> {
+impl<G: Group, const NUM_WINDOWS: usize, const WINDOW_SIZE: usize> BoweHopwoodPedersenCRH<G, NUM_WINDOWS, WINDOW_SIZE> {
     pub fn create_generators<R: Rng>(rng: &mut R) -> Vec<Vec<G>> {
-        let mut generators = Vec::with_capacity(S::NUM_WINDOWS);
-        for _ in 0..S::NUM_WINDOWS {
-            let mut generators_for_segment = Vec::with_capacity(S::WINDOW_SIZE);
+        let mut generators = Vec::with_capacity(NUM_WINDOWS);
+        for _ in 0..NUM_WINDOWS {
+            let mut generators_for_segment = Vec::with_capacity(WINDOW_SIZE);
             let mut base = G::rand(rng);
-            for _ in 0..S::WINDOW_SIZE {
+            for _ in 0..WINDOW_SIZE {
                 generators_for_segment.push(base);
                 for _ in 0..4 {
                     base.double_in_place();
@@ -57,11 +55,13 @@ impl<G: Group, S: PedersenSize> BoweHopwoodPedersenCRH<G, S> {
     }
 }
 
-impl<G: Group, S: PedersenSize> CRH for BoweHopwoodPedersenCRH<G, S> {
+impl<G: Group, const NUM_WINDOWS: usize, const WINDOW_SIZE: usize> CRH
+    for BoweHopwoodPedersenCRH<G, NUM_WINDOWS, WINDOW_SIZE>
+{
     type Output = G;
-    type Parameters = PedersenCRHParameters<G, S>;
+    type Parameters = PedersenCRHParameters<G, NUM_WINDOWS, WINDOW_SIZE>;
 
-    const INPUT_SIZE_BITS: usize = PedersenCRH::<G, S>::INPUT_SIZE_BITS;
+    const INPUT_SIZE_BITS: usize = PedersenCRH::<G, NUM_WINDOWS, WINDOW_SIZE>::INPUT_SIZE_BITS;
 
     fn setup<R: Rng>(rng: &mut R) -> Self {
         fn calculate_num_chunks_in_segment<F: PrimeField>() -> usize {
@@ -77,7 +77,7 @@ impl<G: Group, S: PedersenSize> CRH for BoweHopwoodPedersenCRH<G, S> {
         }
 
         let maximum_num_chunks_in_segment = calculate_num_chunks_in_segment::<G::ScalarField>();
-        if S::WINDOW_SIZE > maximum_num_chunks_in_segment {
+        if WINDOW_SIZE > maximum_num_chunks_in_segment {
             panic!(
                 "Bowe-Hopwood hash must have a window size resulting in scalars < (p-1)/2, \
                  maximum segment size is {}",
@@ -87,9 +87,9 @@ impl<G: Group, S: PedersenSize> CRH for BoweHopwoodPedersenCRH<G, S> {
 
         let time = start_timer!(|| format!(
             "BoweHopwoodPedersenCRH::Setup: {} segments of {} 3-bit chunks; {{0,1}}^{{{}}} -> G",
-            S::NUM_WINDOWS,
-            S::WINDOW_SIZE,
-            S::WINDOW_SIZE * S::NUM_WINDOWS * BOWE_HOPWOOD_CHUNK_SIZE
+            NUM_WINDOWS,
+            WINDOW_SIZE,
+            WINDOW_SIZE * NUM_WINDOWS * BOWE_HOPWOOD_CHUNK_SIZE
         ));
         let bases = Self::create_generators(rng);
         end_timer!(time);
@@ -107,22 +107,18 @@ impl<G: Group, S: PedersenSize> CRH for BoweHopwoodPedersenCRH<G, S> {
     fn hash(&self, input: &[u8]) -> Result<Self::Output, CRHError> {
         let eval_time = start_timer!(|| "BoweHopwoodPedersenCRH::Eval");
 
-        if (input.len() * 8) > S::WINDOW_SIZE * S::NUM_WINDOWS {
-            return Err(CRHError::IncorrectInputLength(
-                input.len(),
-                S::WINDOW_SIZE,
-                S::NUM_WINDOWS,
-            ));
+        if (input.len() * 8) > WINDOW_SIZE * NUM_WINDOWS {
+            return Err(CRHError::IncorrectInputLength(input.len(), WINDOW_SIZE, NUM_WINDOWS));
         }
-        assert!(S::WINDOW_SIZE <= MAX_WINDOW_SIZE);
-        assert!(S::NUM_WINDOWS <= MAX_NUM_WINDOWS);
+        assert!(WINDOW_SIZE <= MAX_WINDOW_SIZE);
+        assert!(NUM_WINDOWS <= MAX_NUM_WINDOWS);
 
         // overzealous but stack allocation
         let mut buffer = [0u8; MAX_WINDOW_SIZE * MAX_NUM_WINDOWS / 8 + BOWE_HOPWOOD_CHUNK_SIZE + 1];
         buffer[..input.len()].copy_from_slice(input);
         let buf_slice = (&buffer[..]).view_bits::<Lsb0>();
 
-        let mut bit_len = S::WINDOW_SIZE * S::NUM_WINDOWS;
+        let mut bit_len = WINDOW_SIZE * NUM_WINDOWS;
         if bit_len % BOWE_HOPWOOD_CHUNK_SIZE != 0 {
             bit_len += BOWE_HOPWOOD_CHUNK_SIZE - (bit_len % BOWE_HOPWOOD_CHUNK_SIZE);
         }
@@ -131,21 +127,21 @@ impl<G: Group, S: PedersenSize> CRH for BoweHopwoodPedersenCRH<G, S> {
 
         assert_eq!(
             self.parameters.bases.len(),
-            S::NUM_WINDOWS,
+            NUM_WINDOWS,
             "Incorrect pp of size {:?} for window params {:?}x{:?}x{}",
             self.parameters.bases.len(),
-            S::WINDOW_SIZE,
-            S::NUM_WINDOWS,
+            WINDOW_SIZE,
+            NUM_WINDOWS,
             BOWE_HOPWOOD_CHUNK_SIZE,
         );
-        assert_eq!(self.parameters.bases.len(), S::NUM_WINDOWS);
+        assert_eq!(self.parameters.bases.len(), NUM_WINDOWS);
         for bases in self.parameters.bases.iter() {
-            assert_eq!(bases.len(), S::WINDOW_SIZE);
+            assert_eq!(bases.len(), WINDOW_SIZE);
         }
         let base_lookup = self.bowe_hopwood_parameters.base_lookup(&self.parameters);
-        assert_eq!(base_lookup.len(), S::NUM_WINDOWS);
+        assert_eq!(base_lookup.len(), NUM_WINDOWS);
         for bases in base_lookup.iter() {
-            assert_eq!(bases.len(), S::WINDOW_SIZE);
+            assert_eq!(bases.len(), WINDOW_SIZE);
         }
         assert_eq!(BOWE_HOPWOOD_CHUNK_SIZE, 3);
 
@@ -154,7 +150,7 @@ impl<G: Group, S: PedersenSize> CRH for BoweHopwoodPedersenCRH<G, S> {
         // for all i. Described in section 5.4.1.7 in the Zcash protocol
         // specification.
         let result = buf_slice[..bit_len]
-            .chunks(S::WINDOW_SIZE * BOWE_HOPWOOD_CHUNK_SIZE)
+            .chunks(WINDOW_SIZE * BOWE_HOPWOOD_CHUNK_SIZE)
             .zip(base_lookup)
             .map(|(segment_bits, segment_generators)| {
                 segment_bits
@@ -178,8 +174,10 @@ impl<G: Group, S: PedersenSize> CRH for BoweHopwoodPedersenCRH<G, S> {
     }
 }
 
-impl<G: Group, S: PedersenSize> From<PedersenCRHParameters<G, S>> for BoweHopwoodPedersenCRH<G, S> {
-    fn from(parameters: PedersenCRHParameters<G, S>) -> Self {
+impl<G: Group, const NUM_WINDOWS: usize, const WINDOW_SIZE: usize>
+    From<PedersenCRHParameters<G, NUM_WINDOWS, WINDOW_SIZE>> for BoweHopwoodPedersenCRH<G, NUM_WINDOWS, WINDOW_SIZE>
+{
+    fn from(parameters: PedersenCRHParameters<G, NUM_WINDOWS, WINDOW_SIZE>) -> Self {
         Self {
             bowe_hopwood_parameters: BoweHopwoodPedersenCRHParameters::new(),
             parameters,
@@ -187,7 +185,9 @@ impl<G: Group, S: PedersenSize> From<PedersenCRHParameters<G, S>> for BoweHopwoo
     }
 }
 
-impl<F: Field, G: Group + ToConstraintField<F>, S: PedersenSize> ToConstraintField<F> for BoweHopwoodPedersenCRH<G, S> {
+impl<F: Field, G: Group + ToConstraintField<F>, const NUM_WINDOWS: usize, const WINDOW_SIZE: usize> ToConstraintField<F>
+    for BoweHopwoodPedersenCRH<G, NUM_WINDOWS, WINDOW_SIZE>
+{
     #[inline]
     fn to_field_elements(&self) -> Result<Vec<F>, ConstraintFieldError> {
         self.parameters.to_field_elements()
@@ -200,21 +200,18 @@ mod tests {
     use rand::SeedableRng;
     use snarkvm_curves::edwards_bls12::EdwardsProjective;
 
-    #[derive(Debug, Clone, PartialEq, Eq, Hash)]
-    pub struct CRHSize;
-
-    impl PedersenSize for CRHSize {
-        const NUM_WINDOWS: usize = 8;
-        const WINDOW_SIZE: usize = 32;
-    }
+    const NUM_WINDOWS: usize = 8;
+    const WINDOW_SIZE: usize = 32;
 
     #[test]
     fn test_bowe_pedersen() {
         let mut rng = rand_xorshift::XorShiftRng::seed_from_u64(23453245);
-        let parameters = <BoweHopwoodPedersenCRH<EdwardsProjective, CRHSize> as CRH>::setup(&mut rng);
+        let parameters = <BoweHopwoodPedersenCRH<EdwardsProjective, NUM_WINDOWS, WINDOW_SIZE> as CRH>::setup(&mut rng);
         let input = vec![127u8; 32];
 
-        let output = <BoweHopwoodPedersenCRH<EdwardsProjective, CRHSize> as CRH>::hash(&parameters, &input).unwrap();
+        let output =
+            <BoweHopwoodPedersenCRH<EdwardsProjective, NUM_WINDOWS, WINDOW_SIZE> as CRH>::hash(&parameters, &input)
+                .unwrap();
         assert_eq!(
             &*output.to_string(),
             "GroupAffine(x=232405123812771034726439972860096518067116445442313271493943612938654881935, y=752634260468672343124870935373206613671657768711738358314821821547485346646)"
